#include "OneWire.h"

uint8_t dspin;
 
OneWire::OneWire(uint8_t pin)
{
  dspin = pin;
  pinMode(dspin, intype);
}

// Perform the onewire reset function.  We will wait up to 250uS for
// the bus to come high, if it doesn't then it is broken or shorted
// and we return a 0;
//
// Returns 1 if a device asserted a presence pulse, 0 otherwise.
//
uint8_t OneWire::reset(void)
{
	uint8_t r;
	uint8_t retries = 125;

  pinMode(dspin, intype);
	// wait until the wire is high... just in case
	do {
		if (--retries == 0) return 0;
		delayMicroseconds(2);
	} while ( !digitalRead(dspin));

  digitalWrite(dspin,0);
  pinMode(dspin, OUTPUT);
	delayMicroseconds(480);
  pinMode(dspin, intype);
	delayMicroseconds(70);
	r = !digitalRead(dspin);
	delayMicroseconds(410);
	return r;
}

//
// Write a bit. Port and bit is used to cut lookup time and provide
// more certain timing.
//
void OneWire::write_bit(uint8_t v)
{
	if (v & 1) 
	{
    digitalWrite(dspin,0);
    pinMode(dspin, OUTPUT);
		delayMicroseconds(10);
    digitalWrite(dspin,1);
		delayMicroseconds(55);
	} else {
    digitalWrite(dspin,0);
    pinMode(dspin, OUTPUT);
		delayMicroseconds(65);
    digitalWrite(dspin,1);
		delayMicroseconds(5);
	}
}

//
// Read a bit. Port and bit is used to cut lookup time and provide
// more certain timing.
//
uint8_t OneWire::read_bit(void)
{
	uint8_t r;

  digitalWrite(dspin,0);
  pinMode(dspin, OUTPUT);
	delayMicroseconds(3);
  pinMode(dspin, intype);
	delayMicroseconds(10);
	r = digitalRead(dspin);
	delayMicroseconds(53);
	return r;
}

//
// Write a byte. The writing code uses the active drivers to raise the
// pin high, if you need power after the write (e.g. DS18S20 in
// parasite power mode) then set 'power' to 1, otherwise the pin will
// go tri-state at the end of the write to avoid heating in a short or
// other mishap.
//
void OneWire::write(uint8_t v) 
{
  uint8_t bitMask;
  for (bitMask = 0x01; bitMask; bitMask <<= 1) 
	    OneWire::write_bit( (bitMask & v)?1:0);
}

void OneWire::write_bytes(const uint8_t *buf, uint16_t count) 
{
  for (uint16_t i = 0 ; i < count ; i++)
    write(buf[i]);
}

//
// Read a byte
//
uint8_t OneWire::read() 
{
  uint8_t bitMask;
  uint8_t r = 0;

  for (bitMask = 0x01; bitMask; bitMask <<= 1) 
  {
	  if ( OneWire::read_bit()) r |= bitMask;
  }
  return r;
}

void OneWire::read_bytes(uint8_t *buf, uint16_t count) 
{
  for (uint16_t i = 0 ; i < count ; i++)
    buf[i] = read();
}

//
// Do a ROM select
//
void OneWire::select(const uint8_t rom[8])
{
    uint8_t i;
    write(0x55);           // Choose ROM
    for (i = 0; i < 8; i++) write(rom[i]);
}

//
// Do a ROM skip
//
void OneWire::skip()
{
    write(0xCC);           // Skip ROM
}
