/*
*/

#include "OneWire.h"

#define Sensor_IN A0 //14
#define PWM_CT1A   9 //OC1A
#define PWM_CT1B  10 //OC1B
#define set_temp 30;
#define maxpwm 320
#define minpwm   0

OneWire  ds(Sensor_IN);  

void ct1_init()
{
  digitalWrite(PWM_CT1A, LOW);
  pinMode(PWM_CT1A, OUTPUT);
  digitalWrite(PWM_CT1B, LOW);
  pinMode(PWM_CT1B, OUTPUT);
  
  TCCR1A = (1 << COM1A1)|(1 << COM1B1)|(1 << WGM11); 
  TCCR1B = (1 << WGM13)|(1 << CS10);  
  ICR1  = maxpwm;   // aiming for 25kHz
  OCR1A = 10;   // set the PWM duty cycle
  OCR1B = 10;   // set the PWM duty cycle
}

void setup() 
{
  pinMode(LED_BUILTIN, OUTPUT);
  digitalWrite(LED_BUILTIN, HIGH);
  Serial.begin(115200);
  delay(500);

  ct1_init();
}

byte data[12];
int16_t pwm = minpwm;
int P,I;

void loop() 
{
  byte i;
  byte addr[8];
  int celsius,diff,pi;
  byte res;

  res = ds.reset();
  if (res != 1) 
  {
    Serial.println("No Device");
    digitalWrite(LED_BUILTIN, HIGH);
    delay(1000);
    digitalWrite(LED_BUILTIN, LOW);
  }
  
  ds.skip();
  ds.write(0x44);        // start conversion
  
  delay(1000);     // maybe 750ms is enough, maybe not
  
  ds.reset();
  ds.skip();
  ds.write(0xBE);         // Read Scratchpad
  ds.read_bytes(data,9);

  // Convert the data to actual temperature
  // because the result is a 16 bit signed integer, it should
  // be stored to an "int16_t" type, which is always 16 bits
  // even when compiled on a 32 bit processor.
  int16_t raw = (data[1] << 8) | data[0];
  {
    byte cfg = (data[4] & 0x60);
    // at lower res, the low bits are undefined, so let's zero them
    if      (cfg == 0x00) raw = raw & ~7;  // 9 bit resolution, 93.75 ms
    else if (cfg == 0x20) raw = raw & ~3; // 10 bit res, 187.5 ms
    else if (cfg == 0x40) raw = raw & ~1; // 11 bit res, 375 ms
    // default is 12 bit resolution, 750 ms conversion time
  }
  celsius = raw / 16;
  Serial.print("Temp ");
  Serial.print(celsius);

  diff = celsius - set_temp;
  P = diff;
  I = constrain(I+diff,0,20);

  pi = 30*P + I;
  
  Serial.print(" P ");
  Serial.print(P);
  Serial.print(" I ");
  Serial.print(I);
   
  pwm = constrain(pi,minpwm,maxpwm);
  OCR1A = pwm;
  OCR1B = pwm;
  Serial.print(" pwm ");
  Serial.print(pwm);
  Serial.println();
}
